SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[AccountSubAccountVCCBalanceRecharge] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@Target VARCHAR(50) = NULL,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		IF @User IS NULL OR @User = '' SET @User = 'admin'

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @MOVEMENT_ID VARCHAR(50) = NEWID()
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL
		DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
		DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

		-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL DE LA CUENTA
		SELECT @CURRENCY_CODE = A.[CurrencyCode],
				@CURRENCY_SYMBOL = C.[Symbol]
			FROM [dbo].[Account] A (NOLOCK)
			INNER JOIN [dbo].[Currency] C (NOLOCK)
				ON C.[Code] = A.[CurrencyCode]
			WHERE A.[InstallationId] = @InstallationId 
				AND A.[AccountId] = @AccountId
				AND A.[SubAccountId] IS NULL

		-- USAMOS EL PARAMETRO TARGET PARA IDENTIFICAR A QUE NIVEL SE HARA LA RECARGA
		-- EN EL CASO DE RECARGAR CUENTA NO TENEMOS RESTRICCION SOBRE EL ORIGEN DEL SALDO
		IF @Target = 'ACCOUNT'
		BEGIN
			-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
			IF @PlatformHash IS NULL OR @InstallationId IS NULL
				-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
				RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
				SET @MOVEMENT_DESCRIPTION = CONCAT('ACCOUNT ', @AccountId, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(ACCOUNT ', @AccountId, ') FROM BILLING RECHARGES MODULE)') 

				IF @@ROWCOUNT = 0
					RAISERROR('DATABASE_ERROR_UPDATE_ACCOUNT_BALANCE_FAILED', 16, 16)

				-- EL UNICO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA CUENTA
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
							,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
							,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] ,[CreatedByUserId])
					SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'CHARGE'
							,1 ,1 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
							,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,GETUTCDATE() ,@User
						FROM [dbo].[AccountBalance] AB 
						WHERE [ID] = @ACCOUNT_ID
						
				-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
				UPDATE [dbo].[AccountBalance] 
					SET [IsLocked] = 0, 
						[TotalBalance] = [TotalBalance] + @Balance, 
						[AvailableBalance] = [AvailableBalance] + @Balance,
						[LastModifiedByUserId] = @User,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [ID] = @ACCOUNT_ID

			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- EN EL CASO DE SUBCUENTA DEBEMOS VALIDAR QUE EL SALDO A INSERTAR SEA MENOR O IGUAL AL DISPONIBLE DE SU 
		-- PADRE INMEDIATO(CUENTA)
		ELSE IF @TARGET = 'SUBACCOUNT'
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
				SELECT @SUBACCOUNT_ID = AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] = @SubAccountId

				-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
				IF @SUBACCOUNT_ID IS NOT NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @ACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance
	
						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
						-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> SUBACCOUNT FREE)
						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('SUBACCOUNT ', @SubAccountId, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM FREE WALLET(ACCOUNT ', @AccountId, ')') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
									,1 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance, GETUTCDATE() ,@User
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @ACCOUNT_ID
									
						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
							,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
							,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,'FREE'
									,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
									,@Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance 
									,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
									,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AssignedBalance], AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @ACCOUNT_ID

						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)
			
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[TotalBalance] = [TotalBalance] + @Balance,
								[AvailableBalance] = [AvailableBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @SUBACCOUNT_ID

					END 
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
					RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- EN EL CASO DE VCC DEBEMOS VALIDAR QUE EL SALDO A INSERTAR SEA MENOR O IGUAL AL DISPONIBLE DE SU PADRE 
		-- INMEDIATO(CUENTA O SUBCUENTA)
		ELSE IF @TARGET = 'VCC'
		BEGIN 
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC', 16, 16)

				-- VALIDAMOS QUE EL VCC TIENE EL WALLET FREE EN ACCOUNTVCCBALANCE
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
						WHERE AVB.[InstallationId] = @InstallationId 
							AND AVB.[AccountId] = @AccountId
							AND AVB.[VCC] = @VCC
							AND AVB.[SystemService] = 'FREE'
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC_WALLET', 16, 16)
				
				-- SI SUBCUENTA ES NULO EL PADRE ES CUENTA
				IF @SubAccountId IS NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @ACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN LA TABLA DE 
						-- MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> VCC REE)
					
						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ')') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
									,1 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
									,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB (NOLOCK)
								WHERE [ID] = @ACCOUNT_ID
								
						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA AL VCC
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[VCC] ,[Wallet]
							,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
							,[Amount],[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,@VCC ,'FREE'
									,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
									,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] +  @Balance ,AVB.[CountableBalance] ,AVB.[CountableBalance] + @Balance 
									,GETUTCDATE() ,@User
								FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
								WHERE AVB.[InstallationId] = @InstallationId 
									AND AVB.[AccountId] = @AccountId
									AND AVB.[VCC] = @VCC
									AND AVB.[SystemService] = 'FREE'

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
									,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB (NOLOCK)
								WHERE [ID] = @ACCOUNT_ID
				
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- ACTUALIZAMOS EL SALDO FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
						UPDATE [dbo].[AccountVCCBalance] 
							SET [AvailableBalance] = [AvailableBalance] + @Balance,
								[CountableBalance] = [CountableBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_VCC_BALANCA_UPDATE_FAILED', 16, 16)

					END
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

				END 
				-- SI SUBCUENTA ES DIFERENTE DE NULO EL PADRE ES SUBCUENTA
				ELSE
				BEGIN
					-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
					SELECT @SUBACCOUNT_ID = AB.[ID] 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[PlatformHash] = @PlatformHash
							AND AB.[InstallationId] = @InstallationId 
							AND AB.[AccountId] = @AccountId
							AND AB.[SubAccountId] = @SubAccountId

					-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
					IF @SUBACCOUNT_ID IS NOT NULL
					BEGIN
						-- VERIFICAMOS QUE LA SUBCUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
						IF 1 = (SELECT 1 
								FROM [dbo].[AccountBalance] AB 
								WHERE AB.[ID] = @SUBACCOUNT_ID
									AND AB.[AvailableBalance] >= @Balance) 
						BEGIN
							-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
							-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
							UPDATE [dbo].[AccountBalance] 
								SET [IsLocked] = 1,
									[LastModifiedByUserId] = @User,
									[LastModifiedDate] = GETUTCDATE(),
									[TimeStamp] = GETUTCDATE() 
								WHERE [ID] = @SUBACCOUNT_ID
									AND [AvailableBalance] >= @Balance
	
							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

							-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
							-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SUBACCOUNT FREE -> VCC FREE)

							-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
							SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(SUBACCOUNT ', @SubAccountId, ')') 

							-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA SUBCUENTA
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
										([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
										,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
										,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
										,[CreatedDate] ,[CreatedByUserId])
								SELECT @MOVEMENT_ID, @InstallationId, @AccountId, @SubAccountId, 'FREE'
										,'DISCOUNT' ,1 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
										,@Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance 
										,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID
									
							-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DEL VCC
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PostAvailableBalance] ,[PostCountableBalance] ,[CreatedDate])
							VALUES
								(@MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,@VCC
								,'FREE' ,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,@Balance ,@Balance ,GETUTCDATE())

							-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA SUBCUENTA
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
										([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
										,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
										,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
										,[CreatedDate] ,[CreatedByUserId])
								SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,[SubAccountId] ,'ASSIGNED'
										,'CHARGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
										,@Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance 
										,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID
			
							-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
							UPDATE [dbo].[AccountBalance] 
								SET [IsLocked] = 0, 
									[AvailableBalance] = [AvailableBalance] - @Balance,
									[AssignedBalance] = [AssignedBalance] + @Balance
								WHERE [ID] = @SUBACCOUNT_ID
									AND [AvailableBalance] >= @Balance

							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

							-- ACTUALIZAMOS EL SALDO FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
							UPDATE [dbo].[AccountVCCBalance] 
								SET [AvailableBalance] = [AvailableBalance] + @Balance,
									[CountableBalance] = [CountableBalance] + @Balance,
									[LastModifiedByUserId] = @User,
									[LastModifiedDate] = GETUTCDATE(),
									[TimeStamp] = GETUTCDATE() 
								WHERE [InstallationId] = @InstallationId 
									AND [AccountId] = @AccountId
									AND [VCC] = @VCC
									AND [SystemService] = 'FREE'

							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_VCC_BALANCA_UPDATE_FAILED', 16, 16)

						END 
						ELSE 
							-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
					END
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
						RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
				END
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END 
		ELSE 
			-- LANZAMOS MENSAJE QUE SE INGRESO UN TARGET NO VALIDO
			RAISERROR('DATABASE_ERROR_INVALID_TARGET', 16, 16)

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
