SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[AddAccountVCCProduct] (
	@InstallationId Varchar(100) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@VCC Varchar(50) = NULL,
	@Product Varchar(50) = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE BUSCA LOS SERVICIOS DEL SISTEMA ASOCIADOS AL VCC INGRESADO E INSERTA LOS FALTANTES QUE EXISTEN EN LA TABLA SYSTEM_SERVICE
	SET XACT_ABORT, NOCOUNT ON;

    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
        BEGIN TRANSACTION
		
		-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI EXISTEN LICENCIAS DISPONIBLES
		DECLARE @AVAILABLE_LICENSE BIT

		-- ASIGNAMOS FALSE POR DEFECTO
		SET @AVAILABLE_LICENSE = 0

		-- ACTUALIZAMOS EL VALOR SEGUN TENGAMOS LICENCIAS DISPONIBLES O NO
		SELECT @AVAILABLE_LICENSE = CASE WHEN PL.[TotalLicense] > PL.[AssignedLicense] THEN  1 END 
			FROM dbo.[PlatformLicense] PL 
			WHERE PL.[InstallationId] = @InstallationId 
				AND PL.[Product] = @Product

		-- SI TENEMOS LICENCIAS DISPONIBLES CONTINUAMOS
		IF @AVAILABLE_LICENSE = 1
		BEGIN
			-- DECLARAMOS UNA VARIABLE PARA PARA ALMACENAR EL USUARIO DE LA CUENTA
			DECLARE @ACCOUNT_USER VARCHAR(50)

			-- DECLARAMOS UNA VARIABLE PARA PARA ALMACENAR LA FECHA DE EXPIRACION DE LA LICENCIA DE PRODUCTO
			DECLARE @PLATFORM_LICENSE_EXPIRATION_DATE DATE 

			-- RECUPERAMOS LA FECHA DE EXPIRACION DE LA CUENTA
			-- SI LA SUBCUENTA ES DIFERENTE DE NULO
			IF (@SubAccountId IS NOT NULL)
			BEGIN
				SELECT @ACCOUNT_USER = A.[User], 
								@PLATFORM_LICENSE_EXPIRATION_DATE = PL.[ExpirationDate]
					FROM dbo.[Account] A (NOLOCK)
					LEFT JOIN dbo.[PlatformLicense] PL (NOLOCK)
						ON PL.[PlatformHash] = A.[PlatformHash]
							AND PL.[InstallationId] = A.[InstallationId]
					WHERE A.[InstallationId] = @InstallationId 
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] = @SubAccountId
						AND A.[Enabled] = 1
						AND PL.[Product] = @Product
			END

			-- SI LA SUBCUENTA ES NULO
			ELSE
			BEGIN
				SELECT @ACCOUNT_USER = A.[User], 
						@PLATFORM_LICENSE_EXPIRATION_DATE = PL.[ExpirationDate]
					FROM dbo.[Account] A (NOLOCK)
					LEFT JOIN dbo.[PlatformLicense] PL (NOLOCK)
						ON PL.[PlatformHash] = A.[PlatformHash]
							AND PL.[InstallationId] = A.[InstallationId]
					WHERE A.[InstallationId] = @InstallationId 
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] IS NULL
						AND A.[Enabled] = 1
						AND PL.[Product] = @Product
			END

			-- PROCEDEMOS A BLOQUEAR TEMPORALMENTE EL REGISTRO HASTA TERMINAR DE AGREGAR EL PRODUCTO
			UPDATE dbo.[PlatformLicense] 
				SET [IsLocked] = 1,
					[LastModifiedByUserId] = @ACCOUNT_USER,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [Product] = @Product

			-- VERIFICAMOS SI EL INSERT PROCESIO CORRECTAMENTE
			IF @@ROWCOUNT = 0 
				RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_UPDATE_FAILED', 16, 16)
			
			-- VERIFICAMOS NUEVAMENTE QUE AUN TENEMOS LICENCIAS DISPONIBLES 
			SELECT @AVAILABLE_LICENSE = CASE WHEN PL.[TotalLicense] > PL.[AssignedLicense] THEN  1 END 
				FROM dbo.[PlatformLicense] PL 
				WHERE PL.[InstallationId] = @InstallationId 
					AND PL.[Product] = @Product
					AND PL.[IsLocked] = 1

			-- SI TENEMOS LICENCIAS DISPONIBLES CONTINUAMOS A AGREGAR EL PRODUCTO PARA EL VCC
			IF @AVAILABLE_LICENSE = 1
			BEGIN
				-- SI YA TENEMOS UN REGISTRO DEL VCC CON EL PRODUCTO ACTUAL, LO ACTUALIZAMOS A ENABLED 1
				IF EXISTS (
					SELECT 1 
						FROM [dbo].[PlatformLicenseUsage] PLU (NOLOCK)
						WHERE PLU.[InstallationId] = @InstallationId
							AND PLU.[AccountId] = @AccountId
							AND PLU.[VCC] = @VCC
							AND PLU.[Product] = @Product
				)
				BEGIN
					UPDATE [dbo].[PlatformLicenseUsage] 
						SET [Enabled] = 1,
							-- [AccountId] = @AccountId,
							-- [SubAccountId] = @SubAccountId,
							[LicenseActivationDate] = GETUTCDATE(),
							[LicenseExpirationDate] = @PLATFORM_LICENSE_EXPIRATION_DATE,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByUserId] = @ACCOUNT_USER
						WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [Product] = @Product
					-- LANZAMOS MENSAJE DE ERROR SI EL UPDATE FALLO
					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_USAGE_UPDATE_FAILED', 16, 16)

				END
				ELSE
				BEGIN
					-- SI YA TENEMOS UN REGISTRO DEL VCC CON EL PRODUCTO ACTUAL, HACEMOS EL INSERT A LA TABLA
					INSERT INTO [dbo].[PlatformLicenseUsage] ([ID]
							   ,[InstallationId]
							   ,[AccountId]
							   ,[SubAccountId]
							   ,[Product]
							   ,[VCC]
							   ,[LicenseActivationDate]
							   ,[LicenseExpirationDate]
							   ,[CreatedDate]
							   ,[CreatedByUserId]
							   ,[TimeStamp])
						VALUES (NEWID()
								,@InstallationId 
								,@AccountId
								,@SubAccountId
								,@Product
								,@VCC
								,GETUTCDATE()
								,@PLATFORM_LICENSE_EXPIRATION_DATE
								,GETUTCDATE()
								,@ACCOUNT_USER
								,GETUTCDATE()
								)
					-- VERIFICAMOS SI EL INSERT PROCESIO CORRECTAMENTE
					IF @@ROWCOUNT = 0 
						RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_USAGE_INSERT_FAILED', 16, 16)
				
				END

				-- ACTUALIZAMOS EL CAMPO [AssignedLicense] DE LA TABLA dbo.[PlatformLicense] Y DESBLOQUEAMOS EL REGISTRO
				UPDATE dbo.[PlatformLicense] 
					SET [AssignedLicense] = [AssignedLicense] + 1,
						[IsLocked] = 0,
						[LastModifiedByUserId] = @ACCOUNT_USER,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [InstallationId] = @InstallationId 
						AND [Product] = @Product
						AND [IsLocked] = 1

				-- VERIFICAMOS SI EL UPDATE PROCESIO CORRECTAMENTE
				IF @@ROWCOUNT = 0 
					RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_UPDATE_FAILED', 16, 16)
				
				--SELECT 1 'result'
			END
			-- SI NO TENEMOS LICENCIAS DISPONIBLES RETORNAMOS 0
			ELSE
			BEGIN
				RAISERROR('DATABABASE_ERROR_NOT_AVAILABLE_LICENSE', 16, 16)
			END
		END
		-- SI NO TENEMOS LICENCIAS DISPONIBLES RETORNAMOS 0
		ELSE
		BEGIN
			RAISERROR('DATABABASE_ERROR_NOT_AVAILABLE_LICENSE', 16, 16)
		END

		COMMIT TRANSACTION;
    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
