SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CloseCancelledBatches] (
	@InstallationId VARCHAR(100) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE CERRAR LOS BATCHES CANCELADOS PARA LIBERAR LOS SALDOS
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		DECLARE RECURRENT_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
		FOR 
			-- OBTENEMOS LOS BATCHES QUE ESTAN CANCELADOS PARA SER CERRADOS
			SELECT DISTINCT ASAVST.[BatchID]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
				WHERE ASAVST.[InstallationId] = @InstallationId
					AND ASAVST.[status] LIKE 'CANCELLED%'
					AND GETUTCDATE() >= DATEADD(HOUR ,1 ,ISNULL(ASAVST.[LasModifiedDate], ASAVST.[TimeStamp]))

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
		DECLARE @CURRENT_BATCH_TO_UPDATE VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN RECURRENT_CURSOR
		FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_BATCH_TO_UPDATE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			DECLARE @UTC_TIME DATETIME = GETUTCDATE()

			BEGIN TRY
				-- CERRAMOS LOS BATCHES CANCELADOS PARA LIBERAR SALDOS
				EXEC [dbo].[AccountSubAccountVCCBatchSummaryClose] @BatchId = @CURRENT_BATCH_TO_UPDATE, @ClosedDate = @UTC_TIME
			END TRY
			BEGIN CATCH
				IF ERROR_MESSAGE() = 'DATABASE_ERROR_LAST_STEP_IS_ALREADY_REACHED'
				BEGIN
					PRINT 'FAILED:' + @CURRENT_BATCH_TO_UPDATE
				END 
			END CATCH

			FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_BATCH_TO_UPDATE
		END
		CLOSE RECURRENT_CURSOR
		DEALLOCATE RECURRENT_CURSOR

		/*
		SELECT DISTINCT *
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			WHERE ASAVST.[status] LIKE 'FINISHED_BY_CANCELLED%'
				AND GETUTCDATE() >= DATEADD(HOUR ,1 ,ISNULL(ASAVST.[LasModifiedDate], ASAVST.[TimeStamp]))
		SELECT ASAVBM.* 
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			INNER JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				ON ASAVBM.[BatchId] = ASAVST.[BatchId]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			WHERE ASAVST.[VCC] = 'allegro' 
				AND ASAVST.[status] LIKE 'CANCELLED%'
				AND GETUTCDATE() >= DATEADD(HOUR ,1 ,ISNULL(ASAVST.[LasModifiedDate], ASAVST.[TimeStamp])) 
			ORDER BY ASAVBM.[TimeStamp] DESC

		SELECT TOP 10 * FROM [dbo].[AccountSubAccountVCCBalanceMovement] (NOLOCK)

		EXEC [dbo].[AccountSubAccountVCCBatchSummaryClose] @BatchId = 'FA60B1AE-D55D-4C30-AA1D-843EDF8248F5', @ClosedDate = GETUTCDATE()

		select * FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) where BatchId = '7CB46E70-3AC7-48D0-9CE0-CD3177D5DBF2'
		select * FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) where BatchId = '00BE99A9-D056-4CCC-AD25-DDA5A7473EC6'
		select * FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) where BatchId = '13E4DD6B-8767-488A-A76E-E6A1B3EE2591'
		select * from [dbo].[AccountSubAccountVCCBalanceMovement] (nolock) where BatchId ='02AD6417-5D94-454F-A205-E261C42C5E8E' order by CurrentStep desc
		--7CB46E70-3AC7-48D0-9CE0-CD3177D5DBF2
		--E16FCA01-A1F2-4AB3-9EEF-5FCF0CD5ABBA
		--00BE99A9-D056-4CCC-AD25-DDA5A7473EC6
		--13E4DD6B-8767-488A-A76E-E6A1B3EE2591
		*/

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
