SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CreateAccountVCC] (
	@PlatformHash Varchar(100) = NULL,
	@InstallationId Varchar(50) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@AccountVCC Varchar(50) = NULL,
	@TranscriptionRatePerHour Decimal(18, 5) = 0, 
	@TranscriptionRatePerSecond Decimal(18, 5) = 0, 
	@SemanticAnalysisRatePerTransaction Decimal(18, 5) = 0,  
	@CognitiveAnalysisRatePerTransaction Decimal(18, 5) = 0, 
	@VCCBalance Decimal(18, 5) = 0, 
	@Product Varchar(250) = NULL,
	@User Varchar(50) = NULL
	)
AS
BEGIN

	SET XACT_ABORT, NOCOUNT ON

    BEGIN TRY
        BEGIN TRANSACTION

		-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
		IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
			RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

       -- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL PRODUCTO RECORRIDO
		DECLARE @VCC_PRODUCT VARCHAR(50)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL SPLIT DE PRODUCTOS
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			SELECT Value FROM [dbo].[split](@Product, ',')

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @VCC_PRODUCT
		WHILE @@FETCH_STATUS = 0
		BEGIN 

			-- EJECUTAMOS EL PROCEDURE QUE INSERTA VCCPRODUCT
			EXEC AddAccountVCCProduct @InstallationId=@InstallationId, @AccountId=@AccountId, @SubAccountId=@SubAccountId, @VCC = @AccountVCC, @Product = @VCC_PRODUCT

			--Do something with Id here
			--PRINT @VCC_PRODUCT
			FETCH NEXT FROM MY_CURSOR INTO @VCC_PRODUCT
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- DECLARAMOS UNA VARIABLE PARA PARA ALMACENAR EL USUARIO DE LA CUENTA
		DECLARE @ACCOUNT_USER VARCHAR(50)
		DECLARE @ACCOUNT_CURRENCY_CODE VARCHAR(10)

		-- RECUPERAMOS LA FECHA DE EXPIRACION DE LA CUENTA
		-- SI LA SUBCUENTA ES DIFERENTE DE NULO
		IF (@SubAccountId IS NOT NULL)
		BEGIN
			SELECT @ACCOUNT_USER = A.[User],
					@ACCOUNT_CURRENCY_CODE = A.[CurrencyCode]
				FROM dbo.[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId
					AND A.[Enabled] = 1
		END

		-- SI LA SUBCUENTA ES NULO
		ELSE
		BEGIN
			SELECT @ACCOUNT_USER = A.[User],
					@ACCOUNT_CURRENCY_CODE = A.[CurrencyCode]
				FROM dbo.[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL
					AND A.[Enabled] = 1
		END

		-- INSERTAMOS EL REGISTRO EN VCC-ACCOUNT
		INSERT INTO [dbo].[AccountVCC]
				   ([InstallationId]
				   ,[AccountId]
				   ,[SubAccountId]
				   ,[VCC]
				   ,[CurrencyCode]
				   ,[TranscriptionRatePerHour]
				   ,[TranscriptionRatePerSecond]
				   ,[SemanticAnalysisRatePerTransaction]
				   ,[CognitiveAnalysisRatePerTransaction]
				   ,[CreatedDate]
				   ,[CreatedByUserId]
				   ,[TimeStamp])
			 VALUES
				   (@InstallationId
				   ,@AccountId
				   ,@SubAccountId
				   ,@AccountVCC
				   ,@ACCOUNT_CURRENCY_CODE
				   ,@TranscriptionRatePerHour
				   ,@TranscriptionRatePerSecond
				   ,@SemanticAnalysisRatePerTransaction
				   ,@CognitiveAnalysisRatePerTransaction 
				   ,GETUTCDATE()
				   ,@ACCOUNT_USER
				   ,GETUTCDATE())
			
		-- LANZAMOS MENSAJE DE ERROR SI EL INSERT FALLO
		IF @@ROWCOUNT = 0
			RAISERROR('DATABASE_ERROR_CREATE_ACCOUNTVCC_FAILED', 16, 16)

		-- EJECUTAMOS EL SP QUE INSERTAR/COMPLETA LOS SERVICIOS DEL SISTEMA PARA EL VCC
		EXEC [dbo].[CreateAccountVCCSystemService] @InstallationId, @AccountId, @SubAccountId, @AccountVCC, @User

		-- EJECUTAMOS EL SP QUE INSERTA EL PRIMER SALDO AL VCC EN LA TABLA ACCOUNTVCCBALANCE Y AccountSubAccountVCCBalanceMovement
		EXEC [dbo].[AccountSubAccountVCCBalanceCreate] @PlatformHash, @InstallationId, @AccountId, @SubAccountId, @AccountVCC, @VCCBalance, @User
		
		SELECT 1 'result'
        /* Other good code omitted*/

        COMMIT TRANSACTION;
    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE();
        DECLARE @Severity integer = ERROR_SEVERITY();
        DECLARE @State integer = ERROR_STATE();

        RAISERROR(@Message, @Severity, @State);
        RETURN -1;
    END CATCH;

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
