SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CreateAccountVCCSystemService] (
	@InstallationId Varchar(100) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@VCC Varchar(50) = NULL,
	@User Varchar(50) = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE BUSCA LOS SERVICIOS DEL SISTEMA ASOCIADOS AL VCC INGRESADO E INSERTA LOS FALTANTES QUE EXISTEN EN LA TABLA SYSTEM_SERVICE
	SET XACT_ABORT, NOCOUNT ON;

    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
        BEGIN TRANSACTION

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @InstallationId IS NULL OR @AccountId IS NULL OR @VCC IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)
		
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL SERVICIO DEL SISTEMA RECORRIDO
		DECLARE @SYSTEM_SERVICE VARCHAR(50)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL SPLIT DE SERVICIOS DEL SISTEMA
		DECLARE MY_SYSTEM_SERVICES_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 

			-- BUSCAMOS LOS SERVICIOS DEL SISTEMA QUE NO ESTAN ASOCIADOS AL VCC INGRESADO BUSCANDO POR INSTALLATIONID Y ACCOUNTID
			SELECT SS.[SystemServiceId]		
				FROM dbo.[SystemService] SS (NOLOCK)
				LEFT JOIN dbo.[AccountVCCBalance] AVB (NOLOCK)
					ON AVB.[SystemService] = SS.[SystemServiceId]
						AND AVB.[VCC] = @VCC
						AND AVB.[InstallationId] = @InstallationId
						AND AVB.[AccountId] = @AccountId 
				WHERE AVB.[VCC] IS NULL 
		
		-- ABRIMOS Y RECORREMOS EL ARRAY DE SERVICIOS
		OPEN MY_SYSTEM_SERVICES_CURSOR
		FETCH NEXT FROM MY_SYSTEM_SERVICES_CURSOR INTO @SYSTEM_SERVICE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- DECLARAMOS UNA VARIABLE PARA EL SALDO INICIAL DEL PRODUCTO ACTUAL
			DECLARE @SERVICE_INITIAL_BALANCE DECIMAL(18,5)
		
			-- SETTEAMOS EL SALDO INICIAL DEL PRODUCTO EN 0
			SET @SERVICE_INITIAL_BALANCE = 0

			-- EJECUTAMOS EL INSERT A ACCOUNTVCCBALANCE
			INSERT INTO [dbo].[AccountVCCBalance]
				([InstallationId]
					   ,[AccountId]
					   ,[SubAccountId]
					   ,[VCC]
					   ,[SystemService]
					   ,[CountableBalance]
					   ,[AvailableBalance]
					   ,[CreatedDate]
					   ,[CreatedByUserId]
					   ,[TimeStamp])
				VALUES
				(@InstallationId
					   ,@AccountId
					   ,@SubAccountId
					   ,@VCC
					   ,@SYSTEM_SERVICE
					   ,@SERVICE_INITIAL_BALANCE
					   ,@SERVICE_INITIAL_BALANCE
					   ,GETUTCDATE()
					   ,@User
					   ,GETUTCDATE()
				)

			-- VERIFICAMOS SI EL INSERT PROCESIO CORRECTAMENTE
			IF @@ROWCOUNT = 0 
				RAISERROR('DATABABASE_ERROR_SAVING_VCC_SYSTEM_SERVICE_FAILED', 16, 16)

			--Do something with Id here
			-- PRINT @SYSTEM_SERVICE
			FETCH NEXT FROM MY_SYSTEM_SERVICES_CURSOR INTO @SYSTEM_SERVICE
		END
		CLOSE MY_SYSTEM_SERVICES_CURSOR
		DEALLOCATE MY_SYSTEM_SERVICES_CURSOR

		COMMIT TRANSACTION;
    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH;

END 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
