SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAccountStatusByVcc]
(
	@InstallationId						VARCHAR(100),
	@VCC								VARCHAR(50),
	@Active								BIT = 0
)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH DETENGA EL SP
    BEGIN TRY
        
		-- DECLARAMOS VARIABLES PARA ALMACENAR LA CUENTA Y SUBCUENTA
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL

		-- ASIGNAMOS LOS VALORES CORRESPONDIENTES A LAS VARIABLES
		SELECT @ACCOUNT_ID = AV.[AccountId],
				@SUBACCOUNT_ID = AV.[SubAccountId]
			FROM dbo.[AccountVCC] AV (NOLOCK)
			WHERE AV.[InstallationId] = @InstallationId
				AND AV.[VCC] = @VCC

		-- VERIFICAMOS SI LA CUENTA EXISTE
		IF NOT EXISTS (
			SELECT 1 
				FROM [dbo].[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId
					AND A.[AccountId] = @ACCOUNT_ID
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_ACCOUNT_DOES_NOT_EXIST', 16, 16)

		-- VERIFICAMOS SI LA CUENTA QUE EXISTE ESTA ACTIVA
		IF NOT EXISTS (
			SELECT 1 
				FROM [dbo].[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId
					AND A.[AccountId] = @ACCOUNT_ID
					AND A.[Enabled] = 1
		) AND @Active = 1
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_ACCOUNT_IS_NOT_ACTIVATED', 16, 16)
		
		-- EN CASO SUBACCOUNTID SEA DIFERENTE DE NULL
		IF @SUBACCOUNT_ID IS NOT NULL
		BEGIN
			-- VERIFICAMOS SI LA SUBCUENTA EXISTE
			IF NOT EXISTS (
				SELECT 1 
					FROM [dbo].[Account] A (NOLOCK) 
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @ACCOUNT_ID
						AND A.[SubAccountId] = @SUBACCOUNT_ID
			)
				-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
				RAISERROR('DATABABASE_ERROR_SUBACCOUNT_DOES_NOT_EXIST', 16, 16)

			-- VERIFICAMOS SI LA CUENTA QUE EXISTE ESTA ACTIVA
			IF NOT EXISTS (
				SELECT 1 
					FROM [dbo].[Account] A (NOLOCK) 
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @ACCOUNT_ID
						AND A.[SubAccountId] = @SUBACCOUNT_ID
						AND A.[Enabled] = 1
			) AND @Active = 1
				-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
				RAISERROR('DATABABASE_ERROR_SUBACCOUNT_IS_NOT_ACTIVATED', 16, 16)
		END

		-- RETORNAMOS VALOR 1
		SELECT 1 'result'

    END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
