SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION RETORNA LOS SALDO DE UNA CUENTA O SUBCUENTA DE TODOS LOS PRODUCTOS O UNO ESPECIFICO
CREATE  PROCEDURE [dbo].[GetAccountSubAccountBalance] (
	@InstallationId Varchar(100) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@SystemService Varchar(50) = NULL,
	@SearchType Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	-- SI @AccountId LLEGA COMO NULL SE BUSCA SOBRE TODAS LAS CUENTAS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- SI @SubAccountId LLEGA COMO NULL SE BUSCA SOBRE TODAS LAS SUBCUENTAS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- SI @SystemService LLEGA COMO NULL SE BUSCA SOBRE TODOS LOS PRODUCTOS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- LA VARIABLE @SearchType NOS INDICA SI VAMOS A BUSCAR CUENTA(S) O SUBCUENTA(S)

	IF @SearchType = 'CTA'
	BEGIN 
		-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
		;WITH CTE_SUBACCOUNS_TOTAL_AVAILABLE_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE DE LAS SUBCUENTAS
			SELECT INNER_AB.[InstallationId]								'InstalationId',
					INNER_AB.[AccountId]									'AccountId',
					SUM(INNER_AB.[AvailableBalance])						'SubAccountsTotalAvailableBalance'
				FROM [dbo].[AccountBalance] INNER_AB (NOLOCK) 
				WHERE INNER_AB.[InstallationId] = @InstallationId 
					AND INNER_AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AB.[AccountId] ELSE @AccountId END
					AND INNER_AB.[SubAccountId] IS NOT NULL
				GROUP BY INNER_AB.[InstallationId], INNER_AB.[AccountId]

		),
		CTE_VCCS_TOTAL_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE Y RETENIDO DE LOS VCCS (YA SEAN DIRECTOS O DE UNA SUBCUENTA DE LA CUENTA)
			SELECT INNER_AVB.[InstallationId]								'InstalationId',
					INNER_AVB.[AccountId]									'AccountId',
					CTE_SATAB.[SubAccountsTotalAvailableBalance]			'SubAccountsTotalAvailableBalance',
					SUM(INNER_AVB.[AvailableBalance])						'VCCsTotalAvailableBalance', 
					SUM(INNER_AVB.[RetainedBalance])						'VCCsTotalRetainedBalance' 
				FROM [dbo].[AccountVCCBalance] INNER_AVB (NOLOCK) 
				LEFT JOIN CTE_SUBACCOUNS_TOTAL_AVAILABLE_BALANCE CTE_SATAB 
					ON CTE_SATAB.[InstalationId] = INNER_AVB.[InstallationId]
						AND CTE_SATAB.[AccountId] = INNER_AVB.[AccountId]
				WHERE INNER_AVB.[InstallationId] = @InstallationId 
					AND INNER_AVB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AVB.[AccountId] ELSE @AccountId END
				GROUP BY INNER_AVB.[InstallationId], INNER_AVB.[AccountId], CTE_SATAB.[SubAccountsTotalAvailableBalance]
		)
			
		-- HACEMOS LA BUSQUEDA DE LA CUENTA DONDE SUBCUENTA ES NULL
		SELECT AB.[AccountId]												'accountId', 
				AB.[SubAccountId]											'subaccountId',
				AB.[TotalBalance]											'totalBalance',
				AB.[AvailableBalance]										'availableBalance',
				AB.[AssignedBalance]										'assignedBalance',
				ISNULL(CTE_VTB.[SubAccountsTotalAvailableBalance], 0)		'subAccountsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalAvailableBalance], 0)				'vccsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalRetainedBalance], 0)				'vccsTotalRetainedBalance'
			FROM dbo.[AccountBalance] AB (NOLOCK) 
			LEFT JOIN CTE_VCCS_TOTAL_BALANCE CTE_VTB 
				ON CTE_VTB.[InstalationId] = AB.[InstallationId]
					AND CTE_VTB.[AccountId] = AB.[AccountId]
			WHERE AB.[InstallationId] = @InstallationId 
				AND AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN AB.[AccountId] ELSE @AccountId END
				AND AB.[SubAccountId] IS NULL
	END
	ELSE 
	BEGIN
		-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
		;WITH CTE_VCCS_TOTAL_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE Y RETENIDO DE LOS VCCS DE LA SUBCUENTA
			SELECT INNER_AVB.[InstallationId]								'InstalationId',
					INNER_AVB.[AccountId]									'AccountId',
					INNER_AVB.[SubAccountId]								'SubAccountId',
					SUM(INNER_AVB.[AvailableBalance])						'VCCsTotalAvailableBalance', 
					SUM(INNER_AVB.[RetainedBalance])						'VCCsTotalRetainedBalance' 
				FROM [dbo].[AccountVCCBalance] INNER_AVB (NOLOCK) 
				WHERE INNER_AVB.[InstallationId] = @InstallationId 
					AND INNER_AVB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AVB.[AccountId] ELSE @AccountId END
					AND INNER_AVB.[SubAccountId] = CASE WHEN @SubAccountId IS NULL THEN INNER_AVB.[SubAccountId] ELSE @SubAccountId END
				GROUP BY INNER_AVB.[InstallationId], INNER_AVB.[AccountId], INNER_AVB.[SubAccountId]
		)

		-- HACEMOS LA BUSQUEDA DE LA SUBCUENTA DONDE SUBCUENTA ES DIFERENTE DE NULO
		SELECT AB.[AccountId]												'accountId', 
				AB.[SubAccountId]											'subaccountId',
				AB.[TotalBalance]											'totalBalance',
				AB.[AvailableBalance]										'availableBalance',
				AB.[AssignedBalance]										'assignedBalance',
				ISNULL(CTE_VTB.[VCCsTotalAvailableBalance], 0)				'vccsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalRetainedBalance], 0)				'vccsTotalRetainedBalance'
			FROM dbo.[AccountBalance] AB (NOLOCK) 
			LEFT JOIN CTE_VCCS_TOTAL_BALANCE CTE_VTB 
				ON CTE_VTB.[InstalationId] = AB.[InstallationId]
					AND CTE_VTB.[AccountId] = AB.[AccountId]
					AND CTE_VTB.[SubAccountId] = AB.[SubAccountId]
			WHERE AB.[InstallationId] = @InstallationId 
				AND AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN AB.[AccountId] ELSE @AccountId END
				AND AB.[SubAccountId] = CASE WHEN @SubAccountId IS NULL THEN AB.[SubAccountId] ELSE @SubAccountId END
	END

	--CREATE TABLE #DUMMY ( a Int, b VarChar(42) )
	--SELECT * FROM #DUMMY
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
