SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAccountSubAccountVCCBalanceSummary] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@SystemService VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE TIENE POR FINALIDAD RETORNAR EL SALDO COUNTABLE, DISPONIBLE Y RETENIDO FINAL
	SET NOCOUNT ON

	-- VERIFICAMOS SI EXISTE UNA PLATAFORMA CON EL PLATFORMHASH INGRESADO
	IF NOT EXISTS(
		SELECT P.[PlatformHash] 
			FROM dbo.[Platform] P 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[InstallationId] = @InstallationId
	)
	BEGIN
		-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
		RETURN
	END

	-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
	IF @AccountId IS NULL
	BEGIN
		-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
		RAISERROR('DATABABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)
		RETURN
	END

	-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
	IF @PlatformHash IS NULL OR @InstallationId IS NULL OR @AccountId IS NULL OR @VCC IS NULL
	BEGIN
		-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
		RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)
		RETURN
	END

	-- PARA VALIDAR QUE EXISTE LA CARTERA SOLICITADA BASTA CON CONSULTAR CUENTA Y VCC
	IF @SystemService IS NOT NULL AND NOT EXISTS (
		SELECT 1 
			FROM [dbo].[AccountVCCBalance] AB (NOLOCK)
			WHERE AB.[InstallationId] = @InstallationId 
				AND AB.[AccountId] = @AccountId
				AND AB.[VCC] = @VCC
				AND AB.[SystemService] = @SystemService
	)
	BEGIN
		-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
		RAISERROR('DATABABASE_ERROR_NOT_FOUND_TARGET', 16, 16)
		RETURN
	END

	-- OBTENEMOS EL SUMARIZADO DE LOS SALDOS DEL VCC DE LA TABLA ACCOUNTVCCBALANCE
	SELECT SUM(AVB.[CountableBalance]) 'countableBalance',
			SUM(AVB.[AvailableBalance]) 'availableBalance',
			SUM(AVB.[RetainedBalance]) 'retainedBalance'
		FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
		WHERE AVB.[InstallationId] = @InstallationId
			AND AVB.[AccountId] = @AccountId
			AND AVB.[VCC] = @VCC
			AND AVB.[SystemService] = CASE WHEN @SystemService IS NULL THEN AVB.[SystemService] ELSE @SystemService END 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
