SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchBalanceDetail](
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL,
	@VCC VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL 
			OR @InstallationId IS NULL 
			OR @VCC IS NULL 
			OR @BatchTypeName IS NULL 
			OR @BatchTypeId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @BatchTypeName <> 'TRANSCRIPTION' AND @BatchTypeName <> 'STUDY' AND @BatchTypeName <> 'EVALUATION_PLAN'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPE', 16, 16)

		-- DEFINIMOS EL TIEMPO EN HORAS PARA CERRAR UN BATCH AUTOMATICAMENTE (SI SE MODIFICA, MODIFICAR TAMBIEN EN EL SP GETPLATFORMCURRENTBATCHES)
		DECLARE @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY DECIMAL(18,5) = 72

		--SELECT * FROM [dbo].[AccountSubAccountVCCSummaryTransaction] (NOLOCK)
		--SELECT * FROM [dbo].[AccountSubAccountVCCDetailedTransaction] (NOLOCK) 
		--SELECT * FROM [dbo].[AccountSubAccountVCCBalanceMovement] (NOLOCK) 

		-- HACEMOS UN SELECT A LOS SERVICIOS DEL BATCH, SU AVANCE ACTUAL, MONTO RETENIDO
		SELECT ASAVST.[BatchID]						'batchId',
				ASAVST.[BatchTypeName]				'batchTypeName',
				ASAVST.[BatchTypeId]				'batchTypeId',
				ASAVST.[SystemService]				'systemService', 
				CASE WHEN ASAVST.[SystemService] = 'TRANSCRIPTION' THEN (
						CASE WHEN ASAVST.[DurationTime] - ASAVST.[AlreadyDurationTime] > 0 THEN 
							ASAVST.[DurationTime] - ASAVST.[AlreadyDurationTime]
						ELSE 
							0
						END
					)
					WHEN ASAVST.[SystemService] = 'SEMANTIC_ANALYSIS' THEN ASAVST.[TotalQueries]
					--WHEN ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS' THEN ASAVST.[TotalModels] * ASAVST.[EstimatedRequestsPerModel]
					WHEN ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS' THEN ASAVST.[EstimatedModelRequests]
					END								'estimated',
				CASE WHEN ASAVST.[SystemService] = 'TRANSCRIPTION' THEN SUM(ASAVDT.[ValidDurationTime])
					WHEN ASAVST.[SystemService] = 'SEMANTIC_ANALYSIS' THEN SUM(CASE WHEN ASAVDT.[IsValid] = 1 THEN 1 ELSE 0 END)
					WHEN ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS' THEN SUM(ASAVDT.[TotalValidRequest])
					END								'progress',
				CASE WHEN ASAVST.[SystemService] = 'TRANSCRIPTION' THEN SUM(CASE WHEN ASAVDT.[Description] <> 'SUCCED' THEN ASAVDT.[DurationTime] ELSE 0 END)
					WHEN ASAVST.[SystemService] = 'SEMANTIC_ANALYSIS' THEN SUM(CASE WHEN ASAVDT.[IsValid] = 0 THEN 1 ELSE 0 END)
					WHEN ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS' THEN SUM(ASAVDT.[TotalInvalidRequest])
					END								'failed',
				SUM(ASAVDT.[Rate])					'rated',
				ASAVBM.[Amount]						'retained',
				COUNT(ASAVDT.[BatchId])				'batchDetails',
				ASAVST.[Status]						'batchStatus',
				--AVB.[CountableBalance]				'countableBalance',
				--AVB.[AvailableBalance]				'availableBalance',
				--AVB.[RetainedBalance]				'retainedBalance',
				--CASE WHEN (AVB.[AvailableBalance] - SUM(ASAVDT.[Rate])) >= (ASAVBM.[Amount] - SUM(ASAVDT.[Rate])) THEN 1 ELSE 0 END 'sufficientBalance',
				--CASE WHEN ((SUM(ASAVDT.[Rate]) < ASAVBM.[Amount]) OR (ABS(SUM(ASAVDT.[Rate]) - ASAVBM.[Amount])) < AVB.[AvailableBalance]) THEN 1 ELSE 0 END 'sufficientBalance',
				CASE WHEN ((ISNULL(SUM(ASAVDT.[Rate]), 0) < ASAVBM.[Amount]) 
							OR (ABS(ISNULL(SUM(ASAVDT.[Rate]), 0) - ASAVBM.[Amount])) < AVB.[AvailableBalance]) THEN 1 ELSE 0 END 'sufficientBalance',
				ASAVST.[LasModifiedDate]			'lastModifiedDate',
				--DATEADD(HOUR , 72 , ASAVST.[LasModifiedDate])	'closeDate'
				DATEDIFF(SECOND, CURRENT_TIMESTAMP, DATEADD(HOUR , @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY , ASAVST.[LasModifiedDate])) 'closeDate'
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[BatchId] = ASAVST.[BatchId]
					AND ASAVDT.[BatchTypeName] =  ASAVST.[BatchTypeName]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
				ON ASAVBM.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.BatchId = ASAVST.[BatchId]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountVCCBalance] AVB  (NOLOCK) 
				ON AVB.[InstallationId] = ASAVST.[InstallationId]
					AND AVB.[AccountId] = ASAVBM.[AccountId]
					AND AVB.[VCC] = ASAVST.[VCC]
					AND AVB.[SystemService] = ASAVST.[SystemService]
 			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVST.[VCC] = @VCC
				AND ASAVST.[BatchTypeName] = @BatchTypeName 
				AND ASAVST.[BatchTypeId] = @BatchTypeId
				AND ASAVBM.[Type] = 'RETAIN'
			GROUP BY ASAVST.[BatchId],
					ASAVST.[BatchTypeName],
					ASAVST.[BatchTypeId],
					ASAVST.[SystemService],
					ASAVST.[CreatedDate],
					ASAVST.[DurationTime],
					ASAVST.[AlreadyDurationTime],
					ASAVST.[TotalQueries],
					ASAVST.[TotalModels],
					--ASAVST.[EstimatedRequestsPerModel],
					ASAVST.[EstimatedModelRequests],
					ASAVST.[LasModifiedDate],
					ASAVBM.[Amount],
					ASAVST.[Status],
					AVB.[CountableBalance],
					AVB.[AvailableBalance],
					AVB.[RetainedBalance]
			ORDER BY ASAVST.[CreatedDate] ASC

	END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT ON
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
