SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByAccountSubAccountAvailableBalanceDetail] (
	@PlatformHash VARCHAR(100)=NULL,
	-- @StartDate DATETIME = NULL,
	-- @EndDate DATETIME = NULL,
	@InstallationId VARCHAR(100)=NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	-- @MinAvailableBalance INT = NULL,
	@MaxAvailableBalance INT = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE TIENE POR FINALIDAD RETORNAR EL REPORTE DE RECARGA DE SALDOS
	SET NOCOUNT ON

	-- EN CASO MAXAVAILABLEBALANCE LLEGUE VACIO LO IGUALAMOS A NULL PARA QUE NO SEA CONSIDERADO
	IF @MaxAvailableBalance = '' SET @MaxAvailableBalance = NULL
	-- IF @MinAvailableBalance = '' SET @MinAvailableBalance = NULL

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- VALIDAMOS SI SUBCUENTA ES NULL
		IF @SubAccountId IS NULL
		BEGIN
			-- PARA EL CASO CUANDO SUBACCOUNTID ES NULL

			-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL REPORTE
			WITH CTE_AVAILABLE_BALANCE_REPORT AS (
				-- EL PRIMER SELECT RECUPERA EL SALDO PROPIO DE LA CUENTA (DONDE SUBACCOUNTID = NULL)
				SELECT AB.[InstallationId]											'COLUMN_PLATFORM',
						AB.[AccountId]												'COLUMN_ACCOUNT',
						NULL														'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AB.[AvailableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AB.[InstallationId]
									AND AV.[AccountId] = AB.[AccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AB.[InstallationId]
								AND ASAVBM.[AccountId] = AB.[AccountId]
								AND ASAVBM.[SubAccountId] IS NULL 
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AB.[InstallationId]
							AND A.[AccountId] = AB.[AccountId]
							AND (
								A.[SubAccountId] = AB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE (@InstallationId IS NULL OR AB.[InstallationId] = @InstallationId)
						AND (@AccountId IS NULL OR AB.[AccountId] = @AccountId)
					GROUP BY AB.[InstallationId], AB.[AccountId], A.[CurrencyCode], C.[Symbol]
				UNION
				-- EL SEGUNDO SELECT RECUPERA EL SALDO YA ASIGNADO A LAS SUBCUENTAS Y VCC DE LA CUENTA
				SELECT AVB.[InstallationId]											'COLUMN_PLATFORM',
						AVB.[AccountId]												'COLUMN_ACCOUNT',
						NULL														'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AVB.[CountableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AVB.[InstallationId]
									AND AV.[AccountId] = AVB.[AccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AVB.[InstallationId]
								AND ASAVBM.[AccountId] = AVB.[AccountId]
								AND ASAVBM.[SubAccountId] IS NULL 
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AVB.[InstallationId]
							AND A.[AccountId] = AVB.[AccountId]
							AND (
								A.[SubAccountId] = AVB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AVB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE (@InstallationId IS NULL OR AVB.[InstallationId] = @InstallationId)
						AND (@AccountId IS NULL OR AVB.[AccountId] = @AccountId)
					GROUP BY AVB.[InstallationId], AVB.[AccountId], A.[CurrencyCode], C.[Symbol] 
				UNION
				-- EL TERCER SELECT RECUPERA EL SALDO PROPIO DE LAS SUBCUENTAS
				SELECT AB.[InstallationId]											'COLUMN_PLATFORM',
						AB.[AccountId]												'COLUMN_ACCOUNT',
						AB.[SubAccountId]											'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AB.[AvailableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AB.[InstallationId]
									AND AV.[AccountId] = AB.[AccountId]
									AND AV.[SubAccountId] = AB.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AB.[InstallationId]
								AND ASAVBM.[AccountId] = AB.[AccountId]
								AND ASAVBM.[SubAccountId] = AB.[SubAccountId]
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AB.[InstallationId]
							AND A.[AccountId] = AB.[AccountId]
							AND (
								A.[SubAccountId] = AB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE (@InstallationId IS NULL OR AB.[InstallationId] = @InstallationId)
						AND (@AccountId IS NULL OR AB.[AccountId] = @AccountId)
						AND AB.[SubAccountId] IS NOT NULL
					GROUP BY AB.[InstallationId], AB.[AccountId], AB.[SubAccountId], A.[CurrencyCode], C.[Symbol]
				UNION
				-- EL CUARTO SELECT RECUPERA EL SALDO YA ASIGNADO A LOS VCC DE LAS SUBCUENTAS
				SELECT AVB.[InstallationId]											'COLUMN_PLATFORM',
						AVB.[AccountId]												'COLUMN_ACCOUNT',
						AVB.[SubAccountId]											'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AVB.[CountableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AVB.[InstallationId]
									AND AV.[AccountId] = AVB.[AccountId]
									AND AV.[SubAccountID] = AVB.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AVB.[InstallationId]
								AND ASAVBM.[AccountId] = AVB.[AccountId]
								AND ASAVBM.[SubAccountId] = AVB.[SubAccountId]
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AVB.[InstallationId]
							AND A.[AccountId] = AVB.[AccountId]
							AND (
								A.[SubAccountId] = AVB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AVB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE (@InstallationId IS NULL OR AVB.[InstallationId] = @InstallationId)
						AND (@AccountId IS NULL OR AVB.[AccountId] = @AccountId)
						AND AVB.[SubAccountId] IS NOT NULL
					GROUP BY AVB.[InstallationId], AVB.[AccountId], AVB.[SubAccountId], A.[CurrencyCode], C.[Symbol]
			),
			CTE_AVAILABLE_BALANCE_REPORT_2 AS (
				SELECT CTEABR.[COLUMN_PLATFORM],
						CTEABR.[COLUMN_ACCOUNT],
						CTEABR.[COLUMN_SUBACCOUNT],
						CTEABR.[COLUMN_CURRENCY_CODE],
						-- CTEABR.[COLUMN_CURRENCY_SYMBOL],
						SUM(CTEABR.[COLUMN_AVAILABLE_BALANCE]) 'COLUMN_AVAILABLE_BALANCE',
						CTEABR.[COLUMN_RELATED_VCCS],
						CTEABR.[COLUMN_LAST_RECHARGE_DATE]
					FROM CTE_AVAILABLE_BALANCE_REPORT CTEABR
					GROUP BY CTEABR.[COLUMN_PLATFORM], CTEABR.[COLUMN_ACCOUNT], CTEABR.[COLUMN_SUBACCOUNT], CTEABR.[COLUMN_CURRENCY_CODE], CTEABR.[COLUMN_CURRENCY_SYMBOL], CTEABR.[COLUMN_RELATED_VCCS], CTEABR.[COLUMN_LAST_RECHARGE_DATE]
			)

			SELECT *
				FROM CTE_AVAILABLE_BALANCE_REPORT_2 CTEABR2
				WHERE (@MaxAvailableBalance IS NULL OR CTEABR2.[COLUMN_AVAILABLE_BALANCE] <= @MaxAvailableBalance)
				ORDER BY CTEABR2.[COLUMN_ACCOUNT], CTEABR2.[COLUMN_SUBACCOUNT], CTEABR2.[COLUMN_LAST_RECHARGE_DATE]

		END
		ELSE
		BEGIN
			-- PARA EL CASO CUANDO SUBACCOUNTID ES DIFERENTE DE NULL
			WITH CTE_AVAILABLE_BALANCE_REPORT AS (
				-- EL PRIMER SELECT RECUPERA EL SALDO PROPIO DE LAS SUBCUENTAS
				SELECT AB.[InstallationId]											'COLUMN_PLATFORM',
						AB.[AccountId]												'COLUMN_ACCOUNT',
						AB.[SubAccountId]											'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AB.[AvailableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AB.[InstallationId]
									AND AV.[AccountId] = AB.[AccountId]
									AND AV.[SubAccountId] = AB.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AB.[InstallationId]
								AND ASAVBM.[AccountId] = AB.[AccountId]
								AND ASAVBM.[SubAccountId] = AB.[SubAccountId]
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AB.[InstallationId]
							AND A.[AccountId] = AB.[AccountId]
							AND (
								A.[SubAccountId] = AB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE AB.[InstallationId] = @InstallationId
						AND (@AccountId IS NULL OR AB.[AccountId] = @AccountId)
						AND AB.[SubAccountId] = @SubAccountId
					GROUP BY AB.[InstallationId], AB.[AccountId], AB.[SubAccountId], A.[CurrencyCode], C.[Symbol]
				UNION
				-- EL SEGUNDO SELECT RECUPERA EL SALDO YA ASIGNADO A LOS VCC DE LAS SUBCUENTAS
				SELECT AVB.[InstallationId]											'COLUMN_PLATFORM',
						AVB.[AccountId]												'COLUMN_ACCOUNT',
						AVB.[SubAccountId]											'COLUMN_SUBACCOUNT',
						A.[CurrencyCode]											'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						SUM(AVB.[CountableBalance])									'COLUMN_AVAILABLE_BALANCE',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = AVB.[InstallationId]
									AND AV.[AccountId] = AVB.[AccountId]
									AND AV.[SubAccountID] = AVB.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS',
						(SELECT MAX(ASAVBM.[TimeStamp])	
							FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
							WHERE ASAVBM.[InstallationId] = AVB.[InstallationId]
								AND ASAVBM.[AccountId] = AVB.[AccountId]
								AND ASAVBM.[SubAccountId] = AVB.[SubAccountId]
								AND ASAVBM.[VCC]  IS NULL 
								AND ASAVBM.[Wallet] = 'FREE'
								AND ASAVBM.[Type] = 'CHARGE')						'COLUMN_LAST_RECHARGE_DATE'
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
					INNER JOIN [dbo].[Account] A (NOLOCK)
						ON A.[InstallationId] = AVB.[InstallationId]
							AND A.[AccountId] = AVB.[AccountId]
							AND (
								A.[SubAccountId] = AVB.[SubAccountId]
								OR
								(
									A.[SubAccountId] IS NULL
									AND
									AVB.[SubAccountId] IS NULL
								)
							)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = A.[CurrencyCode]
					WHERE AVB.[InstallationId] = @InstallationId
						AND (@AccountId IS NULL OR AVB.[AccountId] = @AccountId)
						AND AVB.[SubAccountId] = @SubAccountId
					GROUP BY AVB.[InstallationId], AVB.[AccountId], AVB.[SubAccountId], A.[CurrencyCode], C.[Symbol]
			),
			CTE_AVAILABLE_BALANCE_REPORT_2 AS (
				SELECT CTEABR.[COLUMN_PLATFORM],
						CTEABR.[COLUMN_ACCOUNT],
						CTEABR.[COLUMN_SUBACCOUNT],
						CTEABR.[COLUMN_CURRENCY_CODE], 
						CTEABR.[COLUMN_CURRENCY_SYMBOL], 
						SUM(CTEABR.[COLUMN_AVAILABLE_BALANCE]) 'COLUMN_AVAILABLE_BALANCE',
						CTEABR.[COLUMN_RELATED_VCCS],
						CTEABR.[COLUMN_LAST_RECHARGE_DATE]
					FROM CTE_AVAILABLE_BALANCE_REPORT CTEABR
					GROUP BY CTEABR.[COLUMN_PLATFORM], CTEABR.[COLUMN_ACCOUNT], CTEABR.[COLUMN_SUBACCOUNT], CTEABR.[COLUMN_CURRENCY_CODE], CTEABR.[COLUMN_CURRENCY_SYMBOL], CTEABR.[COLUMN_RELATED_VCCS], CTEABR.[COLUMN_LAST_RECHARGE_DATE]
			)

			SELECT CTEABR2.[COLUMN_PLATFORM],
						CTEABR2.[COLUMN_ACCOUNT],
						CTEABR2.[COLUMN_SUBACCOUNT],
						CTEABR2.[COLUMN_CURRENCY_CODE], 
						-- CTEABR2.[COLUMN_CURRENCY_SYMBOL], 
						CTEABR2.[COLUMN_AVAILABLE_BALANCE],
						CTEABR2.[COLUMN_RELATED_VCCS],
						CTEABR2.[COLUMN_LAST_RECHARGE_DATE]
				FROM CTE_AVAILABLE_BALANCE_REPORT_2 CTEABR2
				WHERE (@MaxAvailableBalance IS NULL OR CTEABR2.[COLUMN_AVAILABLE_BALANCE] <= @MaxAvailableBalance)
				ORDER BY CTEABR2.[COLUMN_LAST_RECHARGE_DATE]
					
		END

	END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1

    END CATCH

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
