SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAccount] (
	@PlatformHash VARCHAR(100) = NULL,
	@Id VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@CountryCode VARCHAR(10) = NULL,
	@CurrencyCode VARCHAR(10) = NULL,
	@User VARCHAR(50) = NULL,
	@Token VARCHAR(50) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CreatedByUserId VARCHAR(50) = NULL,
	@Balance  DECIMAL(18,5) = 0
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
        BEGIN TRANSACTION
			-- HACEMOS UNA BUSQUEDA DE LA PLATFORM INGRESADA
			IF NOT EXISTS (SELECT P.[PlatformHash]
							FROM dbo.[Platform] P (NOLOCK)
							WHERE P.[PlatformHash] = @PlatformHash)
			BEGIN
				RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
			END

			-- VALIDAMOS QUE CUANDO EL COSTO DE TRANSCRIPCION POR HORA ES MAYOR A 0, SEA MINIMO DE 0.036 (ESTO SE DEBE CUMPLIR EN TODAS LA MONEDAS)
			IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI YA EXISTE UNA CUENTA PRINCIPAL
			DECLARE @MAIN_ACCOUNT_EXISTS BIT

			-- SETTEAMOS FALSE POR DEFECTO
			SET @MAIN_ACCOUNT_EXISTS = 0

			-- DECLARAMOS UNA VARIABLE PARA GUARDAR EL NIVEL DE LA CUENTA
			DECLARE @ACCOUNT_LEVEL INT

			-- SETTEAMOS NIVEL 2 POR DEFECTO(CUENTAS COMUNES)
			SET @ACCOUNT_LEVEL = 2

			-- CONSULTAMOS SI YA EXISTE UNA CUENTA LEVEL 1 PARA ESE PLATFORMHASH
			SELECT @MAIN_ACCOUNT_EXISTS = 1
				FROM [dbo].[Account] A (NOLOCK) 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[AccountLevel] = 1

			-- VERIFICAMOS SI LA CUENTA PRINCIPAL NO EXISTE
			IF @MAIN_ACCOUNT_EXISTS = 0 
			BEGIN	
				-- SETEAMOS NIVEL PARA LA CUENTA
				SET @ACCOUNT_LEVEL = 1
			END
			ELSE 
			BEGIN
				-- SI LA SUBCUENTA ES DIFERENTE DE NULO
				IF @SubAccountId IS NULL
				BEGIN
					-- HACEMOS UNA BUSQUEDA DE LA ACCOUNTID INGRESADA
					IF EXISTS (SELECT A.[AccountId]
								FROM dbo.[Account] A (NOLOCK)
								WHERE A.[PlatformHash] = @PlatformHash 
									AND A.[InstallationId] = @InstallationId
									AND A.[AccountId] =  @AccountId) AND @Id IS NULL
					BEGIN
						-- SI NO SE ENCONTRO LA CUENTA LANZAMOS UN MENSAJE DE ERROR
						RAISERROR('DATABASE_ERROR_ACCOUNT_ALREADY_EXISTS', 16, 16)
					END
					ELSE 
						-- SETEAMOS NIVEL PARA LA SUBCUENTA
						SET @ACCOUNT_LEVEL = 2
				END
				ELSE 
				BEGIN
					-- HACEMOS UNA BUSQUEDA DE LA ACCOUNTID INGRESADA
					IF EXISTS (SELECT A.[AccountId]
								FROM dbo.[Account] A (NOLOCK)
								WHERE A.[PlatformHash] = @PlatformHash 
									AND A.[InstallationId] = @InstallationId
									AND A.[AccountId] =  @AccountId)
					BEGIN
						-- SI ENCONTRO LA CUENTA, VALIDAMOS QUE NO EXISTE LA SUBCUENTA
						IF EXISTS (SELECT A.[AccountId]
									FROM dbo.[Account] A (NOLOCK)
									WHERE A.[PlatformHash] = @PlatformHash 
										AND A.[InstallationId] = @InstallationId
										AND A.[AccountId] =  @AccountId
										AND A.[SubAccountId] = @SubAccountId) AND @Id IS NULL
						BEGIN
							-- SI SE ENCONTRO LA SUBCUENTA LANZAMOS UN MENSAJE DE ERROR
							RAISERROR('DATABASE_ERROR_SUBACCOUNT_ALREADY_EXISTS', 16, 16)
						END
						ELSE 
							-- SETTEAMOS NIVEL 3(PARA SUBCUENTAS)
							SET @ACCOUNT_LEVEL = 3
					END
					ELSE 
						-- SI NO SE ENCONTRO LA CUENTA LANZAMOS UN MENSAJE DE ERROR
						RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
					
				END
			END

			-- RAISERROR('DATABABASE_ERROR_DEMOSTRATION', 16, 16)

			-- SI EL ID ES NULL SE TRATA DE UNA NUEVA CUENTA/SUBCUENTA
			IF @Id IS NULL
			BEGIN
				-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA
				INSERT INTO [dbo].[Account]
						   ([ID]
						   ,[PlatformHash]
						   ,[InstallationId]
						   ,[AccountId]
						   ,[SubAccountId]
						   ,[CountryCode]
						   ,[CurrencyCode]
						   ,[User]
						   ,[Token]
						   ,[AccountLevel]
						   ,[TranscriptionRatePerHour]
						   ,[TranscriptionRatePerSecond]
						   ,[SemanticAnalysisRatePerTransaction]
						   ,[CognitiveAnalysisRatePerTransaction]
						   ,[CreatedDate]
						   ,[CreatedByUserId]
						   ,[TimeStamp])
					 VALUES
						   (NEWID()
						   ,@PlatformHash
						   ,@InstallationId
						   ,@AccountId
						   ,@SubAccountId
						   ,@CountryCode
						   ,@CurrencyCode
						   ,@User
						   ,@Token
						   ,@ACCOUNT_LEVEL
						   ,@TranscriptionRatePerHour
						   ,@TranscriptionRatePerSecond
						   ,@SemanticAnalysisRatePerTransaction
						   ,@CognitiveAnalysisRatePerTransaction
						   ,GETUTCDATE()
						   ,@CreatedByUserId
						   ,GETUTCDATE())

				-- EJECUTAMOS EL SP QUE CREA EL REGISTRO EN ACCOUNTBALANCE Y LOS RESPECTIVOS REGISTROS EN LA TABLA DE MOVIMIENTOS
				EXEC [dbo].[AccountSubAccountVCCBalanceCreate] @PlatformHash, @InstallationId, @AccountId, @SubAccountId, NULL, @Balance = @Balance, @User = @User

			END
			-- SI EL ID ES DIFERENTE DE NULL ESTAMOS EDITANDO UN REGISTRO
			ELSE
			BEGIN
				-- ACTUALIZAMOS EL REGISTRO EN LA TABLA ACCOUNT
				UPDATE [dbo].[Account]
					SET [User] = @User
						   ,[Token] = @Token
						   -- ,[CountryCode] = @CountryCode
						   -- ,[CurrencyCode] = @CurrencyCode
						   ,[TranscriptionRatePerHour] = @TranscriptionRatePerHour
						   ,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
						   ,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
						   ,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
						   ,[LastModifiedDate] = GETUTCDATE()
						   ,[LastModifiedByUserId] = @CreatedByUserId
						   ,[TimeStamp] = GETUTCDATE()
					WHERE [ID] = @Id
				
				-- SI HUBO CERO ACTUALIZACIONES, LANZAMOS EL MENSAJE DE ERROR
				IF @@ROWCOUNT = 0 
				BEGIN
					IF @ACCOUNT_LEVEL = 1 OR @ACCOUNT_LEVEL = 2 RAISERROR('DATABASE_ERROR_ACCOUNT_UPDATE_FAILED', 16, 16)
					IF @ACCOUNT_LEVEL = 3 RAISERROR('DATABASE_ERROR_SUBACCOUNT_UPDATE_FAILED', 16, 16)
				END 

				-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LA CANTIDAD DE VCC ASOCIADOS
				DECLARE @ACCOUNT_VCCS INT = 0

				-- ACTUALIZAMOS LOS VCC QUE ESTAN ASOCIADOS A LA CUENTA Y/O SUBCUENTA ACTUALIZADA
				IF @ACCOUNT_LEVEL = 1 OR @ACCOUNT_LEVEL = 2
				BEGIN
					-- HACEMOS UN COUNT DE TODOS LOS VCC QUE TIENEN CUENTA CON EL VALOR RECIBIDO Y SUBCUENTA NULL
					SELECT @ACCOUNT_VCCS = COUNT(AV.[VCC])
						FROM [dbo].[AccountVCC] AV (NOLOCK) 
						WHERE AV.[InstallationId] = @InstallationId
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] IS NULL

					-- SI ELRESULTADO ES MAYOR A CERO ACTUALIZAMOS LOS VCC
					IF @ACCOUNT_VCCS > 0
						UPDATE [dbo].[AccountVCC] 
							SET [TranscriptionRatePerHour] = @TranscriptionRatePerHour
								,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
								,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
								,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
								,[LastModifiedDate] = GETUTCDATE()
								,[LastModifiedByUserId] = @CreatedByUserId
								,[TimeStamp] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [SubAccountId] IS NULL

				END
				ELSE IF @ACCOUNT_LEVEL = 3 
				BEGIN
					-- HACEMOS UN COUNT DE TODOS LOS VCC QUE TIENEN CUENTA Y SUBCUENTA CON EL VALOR RECIBIDO 
					SELECT @ACCOUNT_VCCS = COUNT(AV.[VCC])
						FROM [dbo].[AccountVCC] AV (NOLOCK) 
						WHERE AV.[InstallationId] = @InstallationId
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] = @SubAccountId

					-- SI ELRESULTADO ES MAYOR A CERO ACTUALIZAMOS LOS VCC
					IF @ACCOUNT_VCCS > 0
						UPDATE [dbo].[AccountVCC] 
							SET [TranscriptionRatePerHour] = @TranscriptionRatePerHour
								,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
								,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
								,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
								,[LastModifiedDate] = GETUTCDATE()
								,[LastModifiedByUserId] = @CreatedByUserId
								,[TimeStamp] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [SubAccountId] = @SubAccountId

				END

			END

			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'
			--SELECT SCOPE_IDENTITY()

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

    -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
