SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAccountCurrency] (
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(100) = NULL,
	@NewCountryCode VARCHAR(10) = NULL,
	@NewCurrencyCode VARCHAR(10) = NULL,
	@ConvertAccountRates BIT = 0,
	@convertSubAccountsRates BIT = 0,
	@NewTranscriptionRatePerHour DECIMAL(18,5) = 0,
	@NewTranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@NewSemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@NewCognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@ChangeValue DECIMAL(18, 5) = 0,
	@UserId VARCHAR(50) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
        BEGIN TRANSACTION

			-- HACEMOS UNA BUSQUEDA DE LA PLATFORM INGRESADA
			IF NOT EXISTS (
				SELECT P.[PlatformHash]
					FROM dbo.[Platform] P (NOLOCK)
					WHERE P.[InstallationId] = @InstallationId
			)
			BEGIN
				RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
			END

			-- VALIDAMOS QUE CUANDO EL COSTO DE TRANSCRIPCION POR HORA ES MAYOR A 0, SEA MINIMO DE 0.036 (ESTO SE DEBE CUMPLIR EN TODAS LA MONEDAS)
			IF 0 < @NewTranscriptionRatePerHour AND @NewTranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- DECLARAMOS VARIABLES AUXILIARES PARA LA CONVERSION
			DECLARE @OLD_CURRENCY_CODE VARCHAR(5) = NULL
			DECLARE @OLD_CURRENCY_SYMBOL VARCHAR(5) = NULL
			DECLARE @NEW_CURRENCY_SYMBOL VARCHAR(5) = NULL
			DECLARE @ACCOUNT_VCC_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_VCC_TOTAL_RETAINED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_TOTAL_ASSIGNED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_VCC_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_VCC_TOTAL_RETAINED_BALANCE DECIMAL(18, 5) = 0

			-- DECLARARAMOS UNA VARIABLE PARA INDICAR LA SUBCUENTA Y VCC ACTUAL QUE ESTAMOS ACTUALIZANDO
			DECLARE @CURRENT_SUBACCOUNT VARCHAR(100) = NULL
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC VARCHAR(100) = NULL
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE VARCHAR(100) = NULL
			-- DECLARAMOS LAS VARIABLES PARA LOS SALDOS DEL SERVICIO DEL SISTEMA
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE DECIMAL(18, 5) = 0
			-- DECLARAMOS LA VARIABLE EN LA QUE ESCRIBIREMOS LA DESCRIPCION DEL MOVIMIENTO
			DECLARE @MOVEMENT_ID VARCHAR(50) = NULL
			DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL

			-- OBTENEMOS EL ACTUAL CURRENCY DE LA CUENTA
			SELECT @OLD_CURRENCY_CODE = A.[CurrencyCode],
					@OLD_CURRENCY_SYMBOL = C.[Symbol]
				FROM [dbo].[Account] A (NOLOCK) 
				INNER JOIN [dbo].[Currency] C (NOLOCK) 
					ON C.[Code] = A.[CurrencyCode]
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId 
					AND A.[SubAccountId] IS NULL
			
			-- OBTENEMOS EL SIMBOLO DEL NUEVO CURRENCY DE LA CUENTA
			SELECT @NEW_CURRENCY_SYMBOL = [Symbol] FROM [dbo].[Currency] WHERE [Code] = @NewCurrencyCode

			-- VALIDAMOS QUE EL CURRENCY ANTIGUO Y EL NUEVO SEAN DIFERENTES
			IF @OLD_CURRENCY_CODE = @NewCurrencyCode 
				RAISERROR('DATABASE_ERROR_NEW_CURRENCY_AND_OLD_CURRENCY_ARE_SAME', 16, 16)

			

			-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNT PARA TODOS
			UPDATE [dbo].[Account]
				SET [CountryCode] = @NewCountryCode,
					[CurrencyCode] = @NewCurrencyCode,
					[LastModifiedByUserId] = @UserId,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId



			-- DECLARAMOS UN CURSOR PARA RECORRER LOS VCC RELACIONADOS A LA CUENTA
			DECLARE RECURRENT_CURSOR_00 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [SubAccountId]
				FROM [AccountBalance] (NOLOCK) 
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId
					AND [TotalBalance] > 0

			-- ABRIMOS Y RECORREMOS EL ARRAY DE SERVICIOS DE SISTEMA DE VCC A ACTUALIZAR
			OPEN RECURRENT_CURSOR_00
			FETCH NEXT FROM RECURRENT_CURSOR_00 INTO @CURRENT_SUBACCOUNT
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- OBTENEMOS LOS SALDOS DE LA CUENTA/SUBCUENTA ACTUAL
				SELECT @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE = [TotalBalance],
						@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE = [AvailableBalance],
						@CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE = [AssignedBalance]
					FROM [AccountBalance] (NOLOCK) 
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)

				-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
				UPDATE [dbo].[AccountBalance]
					SET [TotalBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue
						,[AvailableBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue
						,[AssignedBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)

				-- GENERAMOS UN ID PARA EL ACTUAL MOVIMIENTO
				SET @MOVEMENT_ID = NEWID()

				-- PASO 1 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO TOTAL DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue, ' TO FREE WALLET (TOTAL_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO CONTABLE
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,1 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue  ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue
							,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 1 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO TOTAL DE LA CUENTA) --



				-- PASO 2 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO DISPONIBLE DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue, ' TO FREE WALLET (AVAILABLE_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO DISPONIBLE
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue ,AB.[TotalBalance] 
							,AB.[TotalBalance] * @ChangeValue ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 2 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO DISPONIBLE DE LA CUENTA) --



				-- PASO 3 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO ASIGNADO DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue, ' TO FREE WALLET (ASSIGNED_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO ASIGNADO
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue ,AB.[AvailableBalance] ,AB.[AvailableBalance] ,AB.[TotalBalance] 
							,AB.[TotalBalance]  ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 3 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO ASIGNADO DE LA CUENTA) --
				
				FETCH NEXT FROM RECURRENT_CURSOR_00 INTO @CURRENT_SUBACCOUNT
			END
			CLOSE RECURRENT_CURSOR_00
			DEALLOCATE RECURRENT_CURSOR_00


			-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNTVCC PARA TODOS
			UPDATE [dbo].[AccountVCC]
				SET [CurrencyCode] = @NewCurrencyCode,
					[LastModifiedByUserId] = @UserId,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId

			-- SI EL FLAG CONVERTIR TARIFAS DE CUENTAS LLEGA 1, ACTUALIZAMOS LAS TARIFAS CON EL VALOR RECIBIDO
			IF @ConvertAccountRates = 1
			BEGIN
				-- ACTUALIZAMOS LAS TARIFAS CONVERTIDAS PARA LA CUENTA
				UPDATE [dbo].[Account]
					SET [TranscriptionRatePerHour] = @NewTranscriptionRatePerHour,
						[TranscriptionRatePerSecond] = @NewTranscriptionRatePerSecond,
						[SemanticAnalysisRatePerTransaction] = @NewSemanticAnalysisRatePerTransaction,
						[CognitiveAnalysisRatePerTransaction] = @NewCognitiveAnalysisRatePerTransaction,
						[LastModifiedByUserId] = @UserId,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							[AccountLevel] = 1
							OR 
							[AccountLevel] = 2
						)
						AND [SubAccountId] IS NULL

				-- ACTUALIZAMOS LAS TARIFAS DE LOS VCCS QUE DESCIENDEN DIRECTAMENTE DE LA CUENTA
				UPDATE AV 
					SET AV.[TranscriptionRatePerHour] = A.[TranscriptionRatePerHour],
						AV.[TranscriptionRatePerSecond] = A.[TranscriptionRatePerSecond],
						AV.[SemanticAnalysisRatePerTransaction] = A.[SemanticAnalysisRatePerTransaction],
						AV.[CognitiveAnalysisRatePerTransaction] = A.[CognitiveAnalysisRatePerTransaction],
						AV.[LastModifiedByUserId] = @UserId,
						AV.[LastModifiedDate] = GETUTCDATE(),
						AV.[TimeStamp] = GETUTCDATE()
					FROM [dbo].[AccountVCC] AV
					INNER JOIN [dbo].[Account] A
						ON A.[InstallationId] = AV.[InstallationId]
							AND A.[AccountId] = AV.[AccountId]
							AND A.[SubAccountId] IS NULL
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] IS NULL

				-- SI EL FLAG CONVERTIR TARIFAS DE SUBCUENTAS LLEGA 1, ACTUALIZAMOS LAS TARIFAS CON EL VALOR PRODUCTO
				IF @convertSubAccountsRates = 1
				BEGIN
					-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNT PARA TODOS LOS SUBACCOUNTS DE LA CUENTA
					UPDATE [dbo].[Account]
						SET [TranscriptionRatePerHour] = CASE 
								WHEN [TranscriptionRatePerHour] * @ChangeValue < 0.036 
									AND [TranscriptionRatePerHour] * @ChangeValue > 0
								THEN 0.036 ELSE [TranscriptionRatePerHour] * @ChangeValue END,
							[TranscriptionRatePerSecond] = CASE
								WHEN [TranscriptionRatePerHour] * @ChangeValue < 0.036 
									AND [TranscriptionRatePerHour] * @ChangeValue > 0
								THEN 0.00001 ELSE [TranscriptionRatePerSecond] * @ChangeValue / 3600 END,
							[SemanticAnalysisRatePerTransaction] = CASE
								WHEN [SemanticAnalysisRatePerTransaction] * @ChangeValue < 0.00001
									AND [SemanticAnalysisRatePerTransaction] * @ChangeValue > 0
								THEN 0.00001 ELSE [SemanticAnalysisRatePerTransaction] * @ChangeValue END,
							[CognitiveAnalysisRatePerTransaction] = CASE
								WHEN [CognitiveAnalysisRatePerTransaction] * @ChangeValue < 0.00001
									AND [CognitiveAnalysisRatePerTransaction] * @ChangeValue > 0
								THEN 0.00001 ELSE [CognitiveAnalysisRatePerTransaction] * @ChangeValue END,
							[LastModifiedByUserId] = @UserId,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE()
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [AccountLevel] = 3
							AND [SubAccountId] IS NOT NULL

					-- ACTUALIZAMOS LAS TARIFAS DE LOS VCCS QUE HEREDAN DE SUBCUENTA
					UPDATE AV 
						SET AV.[TranscriptionRatePerHour] = A.[TranscriptionRatePerHour],
							AV.[TranscriptionRatePerSecond] = A.[TranscriptionRatePerSecond],
							AV.[SemanticAnalysisRatePerTransaction] = A.[SemanticAnalysisRatePerTransaction],
							AV.[CognitiveAnalysisRatePerTransaction] = A.[CognitiveAnalysisRatePerTransaction],
							AV.[LastModifiedByUserId] = @UserId,
							AV.[LastModifiedDate] = GETUTCDATE(),
							AV.[TimeStamp] = GETUTCDATE()
						FROM [dbo].[AccountVCC] AV
						INNER JOIN [dbo].[Account] A
							ON A.[InstallationId] = AV.[InstallationId]
								AND A.[AccountId] = AV.[AccountId]
								AND A.[SubAccountId] = AV.[SubAccountId]
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] IS NOT NULL
				END
			END
			
			-- DECLARAMOS UN CURSOR PARA RECORRER LOS VCC RELACIONADOS A LA CUENTA
			DECLARE RECURRENT_CURSOR_01 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [SubAccountId],
					[VCC]
				FROM [AccountVCC] (NOLOCK) 
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId

			-- ABRIMOS Y RECORREMOS EL ARRAY DE SERVICIOS DE SISTEMA DE VCC A ACTUALIZAR
			OPEN RECURRENT_CURSOR_01
			FETCH NEXT FROM RECURRENT_CURSOR_01 INTO @CURRENT_SUBACCOUNT, @CURRENT_VCC
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- DECLARAMOS OTRO CURSOS PARA RECORRER LOS WALLET DEL VCC RECORRIDO
				DECLARE RECURRENT_CURSOR_02 CURSOR 
				LOCAL STATIC READ_ONLY FORWARD_ONLY
				FOR 
				-- SELECCIONAMOS LOS WALLETS CON SALDO DISPONIBLE DEL VCC ACTUAL
				SELECT [SystemService]
					FROM [AccountVCCBalance] (NOLOCK) 
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)
						AND [VCC] = @CURRENT_VCC
						AND [AvailableBalance] > 0

				-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
				OPEN RECURRENT_CURSOR_02
				FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_VCC_SYSTEM_SERVICE
				WHILE @@FETCH_STATUS = 0
				BEGIN 
					-- OBTENEMOS LOS SALDOS DE SERVICIO DEL SISTEMA ACTUAL
					SELECT @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE = [CountableBalance],
							@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE = [AvailableBalance],
							@CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE = [RetainedBalance]
						FROM [AccountVCCBalance] (NOLOCK) 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)
							AND [VCC] = @CURRENT_VCC
							AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

					-- GENERAMOS UN ID PARA EL ACTUAL MOVIMIENTO
					SET @MOVEMENT_ID = NEWID()


		
				
		
					-- PASO 1 DE 3 INICIO (ACTUALIZAR EL SALDO CONTABLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --

					-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-COUNTABLE_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERTIRMOS EL SALDO CONTABLE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,1 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] ,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue 
								,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [CountableBalance] = @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

					-- PASO 1 DE 3 FIN (ACTUALIZAR EL SALDO CONTABLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --





					-- PASO 2 DE 3 INICIO (ACTUALIZAR EL SALDO DISPONIBLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --
		
					-- SETTEAMOS EL DESCRIPTION PARA EL SEGUNDO PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-AVAILABLE_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERTIMOS EL SALDO DISPONIBLE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,AVB.[CountableBalance] 
								,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [AvailableBalance] = @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE
					
					-- PASO 2 DE 3 FIN (ACTUALIZAR EL SALDO DISPONIBLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --





					-- PASO 3 DE 3 INICIO (ACTUALIZAR EL SALDO RETENIDO DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --
		
					-- SETTEAMOS EL DESCRIPTION PARA EL TERCER PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-RETAINED_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERSION DE SALDOS SE HARA EN SEIS PASOS PARA LA TABLA DE MOVIMIENTOS, PASO 3 AGREGAR EL SALDO RETENIDO NUEVO
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] ,AVB.[CountableBalance] 
								,AVB.[CountableBalance] ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [RetainedBalance] = @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE
		
					-- PASO 3 DE 3 FIN (ACTUALIZAR EL SALDO RETENIDO DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --

		



					FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_VCC_SYSTEM_SERVICE
				END
				CLOSE RECURRENT_CURSOR_02
				DEALLOCATE RECURRENT_CURSOR_02

				FETCH NEXT FROM RECURRENT_CURSOR_01 INTO @CURRENT_SUBACCOUNT, @CURRENT_VCC
			END
			CLOSE RECURRENT_CURSOR_01
			DEALLOCATE RECURRENT_CURSOR_01

			-- RETORNAMOS RESULTADO 1
			SELECT 1 'result'


			

			
		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

    -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
