SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION ACTUALIZA EL ESTADO DE UN BATCHSUMMARY SEGUN SU SYSTEMSERVICE
CREATE PROCEDURE [dbo].[SaveAccountSubAccountVCCBatchSummaryTransactionStatus] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@BatchId VARCHAR(100) = NULL,
	@SystemService VARCHAR(50) = NULL,
	@Status VARCHAR(100) = NULL,
	@UserApi VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL 
			OR @InstallationId IS NULL 
			OR @AccountId IS NULL 
			OR @VCC IS NULL 
			OR @BatchId IS NULL 
			OR @SystemService IS NULL 
			OR @Status IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)
		
		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @SystemService <> 'TRANSCRIPTION' AND @SystemService <> 'SEMANTIC_ANALYSIS' AND @SystemService <> 'COGNITIVE_ANALYSIS'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPE', 16, 16)

		-- VERIFICAMOS QUE EL BATCH EXISTA YA EN LA BASE DE DATOS
		IF NOT EXISTS(
			SELECT ASAVST.[BatchTypeId] 
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
				WHERE ASAVST.[InstallationId] = @InstallationId
					AND ASAVST.[VCC] = @VCC
					AND ASAVST.[BatchId] = @BatchId
					AND ASAVST.[SystemService] = @SystemService
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_BATCH_SUMMARY_NOT_FOUND', 16, 16)

		-- ACTUALIZAMOS EL REGISTRO DE SUMMARYTRANSACTION
		UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction] 
			SET [Status] = @Status,
				[LastModifiedByUserId] = @UserApi,
				[LasModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE [InstallationId] = @InstallationId
				AND [VCC] = @VCC
				AND [BatchId] = @BatchId
				AND [SystemService] = @SystemService
		
		-- VERIFICAMOS SI SE ACTUALIZO EL REGISTRO
		IF @@ROWCOUNT = 0
			RAISERROR('DATABASE_ERROR_BATCH_SUMMARY_STATUS_UPDATE_FAILED', 16, 16)

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

	END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
