SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SavePlatformData] (
	@PlatformHash VARCHAR(100) = NULL,
	@Name VARCHAR(100) = NULL,
	@Description VARCHAR(100) = NULL,
	@CountryCode VARCHAR(10) = NULL,
	@ClientName VARCHAR(100) = NULL,
	@ClientContactName VARCHAR(100) = NULL,
	@ClientContactEmail VARCHAR(50) = NULL,
	@InconcertContactName VARCHAR(100) = NULL,
	@CommercialContactName VARCHAR(100) = NULL,
	@TechnicalContactName VARCHAR(100) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@User VARCHAR(50) = NULL
)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		BEGIN TRANSACTION
			-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
			IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- ACTUALIZAMOS LOS CAMPOS CLIENTCONTACTEMAIL, TRANSCRIPTIONRATEPERHOUR, SEMANTICANALYSISRATEPERTRANSACTION, COGNITIVEANALYSISRATEPERTRANSACTION, LASTMODIFIEDDATE, LASTMODIFIEDBYUSERID y TIMESTAMP DE ACUERDO EL PLATFORMHASH.
			UPDATE [dbo].[PlatformData] SET
				[Name] = @Name,
				[Description] = @Description,
				[CountryCode] = @CountryCode,
				[ClientName] = @ClientName,
				[ClientContactEmail] = @ClientContactEmail,
				[InconcertContactName] = @InconcertContactName,
				[CommercialContactName] = @CommercialContactName,
				[TechnicalContactName] = @TechnicalContactName,
				[TranscriptionRatePerHour] = @TranscriptionRatePerHour,
				[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction,
				[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction,
				[LastModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE PlatformHash = @PlatformHash

			IF(@@ROWCOUNT = 0)
			BEGIN
				-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA.
				INSERT INTO [dbo].[PlatformData]
					   ([PlatformHash]
					   ,[Name]
					   ,[Description]
					   ,[CountryCode]
					   ,[ClientName]
					   ,[ClientContactName]
					   ,[ClientContactEmail]
					   ,[InconcertContactName]
					   ,[CommercialContactName]
					   ,[TechnicalContactName]
					   ,[TranscriptionRatePerHour]
					   ,[TranscriptionRatePerSecond]
					   ,[SemanticAnalysisRatePerTransaction]
					   ,[CognitiveAnalysisRatePerTransaction]
					   ,[CreatedDate]
					   ,[CreatedByUserId]
					   ,[TimeStamp])
				 VALUES
					   (@PlatformHash
					   ,@Name
					   ,@Description
					   ,@CountryCode
					   ,@ClientName
					   ,@ClientContactName
					   ,@ClientContactEmail
					   ,@InconcertContactName
					   ,@CommercialContactName
					   ,@TechnicalContactName
					   ,@TranscriptionRatePerHour
					   ,@TranscriptionRatePerSecond
					   ,@SemanticAnalysisRatePerTransaction
					   ,@CognitiveAnalysisRatePerTransaction
					   ,GETUTCDATE()
					   ,@User
					   ,GETUTCDATE())

			END

			-- INSERTAMOS EL CLIENTNAME EN CASO NO EXISTA
			EXEC [dbo].[SaveClient] @ClientName
	
			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

	 -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
